<?php
/*
Plugin Name: Redirect
Version: 1.7
Plugin URI: http://www.vdfn.altervista.org/
Description: This plugin redirect you to entries using other URLs
Author: Piero VDFN
Author URI: http://www.vdfn.altervista.org/
*/

/**
 * If this constant is set to true, the redirects will act
 * like pages, if there isn't a page with the same ID of the
 * redirect.
 *
 * Disabled by default.
 */
define('REDIRECT_PAGES', false);

/**
 * plugin_redirect
 *
 * This is the main class of Flatpress
 * plugin redirect.
 *
 * It manages the redirect database and the init hook.
 *
 * @author: Piero VDFN <vogliadifarniente@gmail.com>
 * @license: GNU GPL v2
 */
class plugin_redirect {

	/**
	 * Tells if PrettyURLs it's enabled.
	 *
	 * @var bool
	 */
	var $prettyurls=false;

	/**
	 * Tells if RewriteURLs it's enabled.
	 *
	 * @var bool
	 */
	var $rewriteurls=false;

	/**
	 * The file of the plugin's database.
	 *
	 * @var string
	 */
	var $file='';

	/**
	 * The redirect databse.
	 * It will be loaded later.
	 *
	 * @var array
	 */
	var $db=array();

	/**
	 * This is the constructor of the class.
	 *
	 * It tests if the database file is set,
	 * it checks plugins for URL-Rewriting and
	 * it adds hooks.
	 */
	function plugin_redirect() {
		// Set the file where save redirects, if empty
		if(empty($this->file)) {
			$this->file=FP_CONTENT.'plugin_redirect.txt';
		}

		// Check for PrettyURLs
		$this->prettyurls=class_exists('Plugin_PrettyURLs');

		// Check for RewriteURLs
		$this->rewriteurls=class_exists('plugin_rewriteurls');

		// Add hooks handling
		add_filter('init', array(&$this, 'redirect_init'), 15);
		add_filter('prettyurls_unhandled_url', array(&$this, 'prettyurls_hook'));
	}

	/**
	 * The handler for rewritten URLs.
	 *
	 * This function is used as callback for
	 * prettyurls_unhandled_url hook.
	 * It allows to use URL Rewriting.
	 *
	 * @param string $url: the unhandled url
	 * @return string: the cleaned URL.
	 */
	function prettyurls_hook($url) {
		global $fp_params;

		$e=explode('/', $url);
		$e=array_filter($e);

		if(count($e)!=2) {
			return $url;
		}

		if($e[0]=='redirect') {
			$fp_params['redirect']=$e[1];

			if(isset($fp_params['entry'])) {
				unset($fp_params['entry']);
			}

			$url='';
		}

		return $url;
	}

	/**
	 * Handler of the init hook.
	 *
	 * This function is a handler for the init hook.
	 * It sees if the redirect param is set and if it is
	 * replace other params with the ones of the redirect.
	 */
	function redirect_init() {
		// Work just in the index
		if(!defined('MOD_INDEX')) {
			return;
		}

		global $fp_params;

		// Load the database
		$this->loaddb();
		$db=&$this->db;

		// Assign short vars
		$r=@$fp_params['redirect'];
		$p=@$fp_params['page'];

		if(!empty($r)) {
		} elseif(empty($r) && !REDIRECT_PAGES) {
			return;
		} elseif(!empty($p) && !static_exists($p) && isset($db[$p])) {
			$r=$p;
			unset($fp_params['page']);
		} else {
			return;
		}

		// If not existent redirect, raise a 404 error and exit
		if(!isset($db[$r])) {
			$fp_params=array('entry'=>'entry000000-000000');
			return;
		}

		// Redirect exists so overwrite $fp_params with redirect params.
		$fp_params=$db[$r];

		// It's better unset $fp_params['comments'] if they're disabled
		if(@$fp_params['comments']==false && isset($fp_params['comments'])) {
			unset($fp_params['comments']);
		}

		// Show the page or make a HTTP Redirect?
		if(@$fp_params['httpredirect']) {
			if(@$fp_params['comments']) {
				$url=get_comments_link($fp_params['entry']);
			} else {
				$url=get_permalink($fp_params['entry']);
			}

			header('Location: '.$url);
			die;
		}

		return;
	}

	/**
	 * This function makes a redirected URL.
	 *
	 * @param string $id: The redirect ID 
	 * @return string: the URL.
	 */
	function redirect_url($id) {
		// First of all: check for the page mode
		if(!static_exists($id) && REDIRECT_PAGES) {
			return theme_staticlink($id);
		}

		// If we have a rewriter plugin, we'll use this variable later
		$base='';

		// First: no redirect plugin... Let's do it immediately
		if(!$this->prettyurls && !$this->rewriteurls) {

			return BLOG_BASEURL.'?redirect='.$id;

		} elseif($this->prettyurls) {

			// PrettyURLs
			global $plugin_prettyurls;
			$base=$plugin_prettyurls->baseurl;

		} elseif($this->rewriteurls) {

			// RewriteURLs
			global $plugin_rewriteurls;
			$base=$plugin_rewriteurls->baseurl;

		}

		return $base.'redirect/'.$id.'/';
	}

	/**
	 * This function loads the redirect database.
	 *
	 * @param bool $force: reload if already loaded?
	 * @return bool: Succeeded?
	 */
	function loaddb($force=false) {
		if(!empty($this->db) && !$force) {
			return true;
		}

		$this->db=array();

		$data=io_load_file($this->file);

		if($data===false) {
			return false;
		} elseif(trim($data)=='') {
			return true;
		} elseif(!$this->db=unserialize($data)) {
			return false;
		}

		return true;
	}

	/**
	 * This function saves the redirect database.
	 *
	 * @return bool: Succeeded?
	 */
	function savedb() {
		// Delete redirects whose target doesn't exist
		foreach($this->db as $id=>$params) {
			if(!entry_exists($params['entry'])) {
				unset($this->db[$id]);
			}
		}

		return io_write_file($this->file, serialize($this->db));
	}

	/**
	 * Clean a redirect id.
	 *
	 * @param string $id: The redirect id
	 * @return string: The cleaned redirect id
	 */
	function cleanId($id) {
		$id=preg_replace('/[^a-z0-9\\-_\\.]*/', '', $id);
		return $id;
	}
}

global $plugin_redirect;
$plugin_redirect=new plugin_redirect();

// Admin panel
if(class_exists('AdminPanelAction')) {
	/**
	 * The class that manages the Redirect Admin Panel.
	 *
	 * @author: Piero VDFN <vogliadifarniente@gmail.com>
	 * @license: GNU GPL v2
	 */
	class admin_plugin_redirect extends AdminPanelAction {

		/**
		 * Tells to the admin panel what language file
		 * it has to load.
		 *
		 * @var string
		 */
		var $langres='plugin:redirect';

		/**
		 * The allowed commands.
		 *
		 * @var array
		 */
		var $commands=array('edit', 'delete', 'delok'); 

		/**
		 * The allowed events.
		 *
		 * @var array
		 */
		var $events=array('save', 'cancel');

		/**
		 * This function is used as a callback when the panel
		 * is loaded.
		 *
		 * @author: Piero VDFN <vogliadifarniente@gmail.com>
		 */
		function setup() {
			$this->smarty->assign('admin_resource', 'plugin:redirect/admin.plugin.redirect');
			$this->smarty->assign('pburl', plugin_geturl('redirect'));
		}

		/**
		 * Handles the edit command.
		 *
		 * This is the edit command.
		 * It assign the id of the post and then call
		 * admin_plugin_redirect::main.
		 *
		 * @param string $id: the redirect id.
		 * @return integer: The panel redirection mode
		 */
		function doedit($id) {
			// Load the plugin
			global $plugin_redirect;
			$pr=&$plugin_redirect;
			$pr->loaddb();

			if(!isset($pr->db[$id])) {
				$this->smarty->assign('success', -2);
				return 2;
			}

			return $this->main($id);
		}

		/**
		 * Handles the delete command.
		 *
		 * This is the delete command.
		 * It asks for the delete confirmation.
		 *
		 * @param string $id: the redirect id
		 * @return integer: The panel redirection mode
		 */
		function dodelete($id) {
			// Load the plugin
			global $plugin_redirect;
			$pr=&$plugin_redirect;
			$pr->loaddb();

			// If the redirect exsts
			if(isset($pr->db[$id])) {
				$this->smarty->assign('todelete', $id);
				$this->smarty->assign('admin_resource', 'plugin:redirect/admin.plugin.redirect.delete');
				return 0;
			} else {
				// Complains about non existent redirect
				$this->smarty->assign('success', -3);
				return 2;
			}
		}

		/**
		 * Handles the delok command.
		 *
		 * This is the delok command.
		 * It the redirect exists it is deleted.
		 *
		 * @author: Piero VDFN <vogliadifarniente@gmail.com>
		 * @param string $id: the redirect id.
		 */
		function dodelok($id) {
			global $plugin_redirect;
			$pr=&$plugin_redirect;
			$pr->loaddb();

			if(!isset($pr->db[$id])) {
				$s=-3;
			} else {
				unset($pr->db[$id]);
				$s=$pr->savedb() ? 4 : -4;
			}

			$this->smarty->assign('success', $s);
			return 2;
		}

		/**
		 * The main function of the panel.
		 *
		 * @param string $edit: The id of the redirect to edit
		 * @param boolean $nocheck: Don't check if the redirect exists?
		 * @return integer: The panel redirection option
		 */
		function main($edit=null, $nocheck=false) {
			global $plugin_redirect, $lang;

			$pr=&$plugin_redirect;
			$pr->loaddb();

			$smarty=&$this->smarty;

			if(!is_null($edit) && !isset($pr->db[$edit]) && !$nocheck) {
				$smarty->assign('success', -2);
				return 2;
			} elseif(!is_null($edit) && isset($pr->db[$edit])) {
				$smarty->assign('editr', $edit);
			}

			if(!count($pr->db)) {
				// There aren't redirect
				$smarty->assign('noredirect', true);
			} else {
				$smarty->assign('redirects', $pr->db);
			}

			// Don't redirect
			return 0;
		}

		/**
		 * The handler for the Save button.
		 *
		 * @return integer: The panel redirection option
		 */
		function onsave() {
			// Load the plugin
			global $plugin_redirect;
			$pr=&$plugin_redirect;
			$pr->loaddb();

			// The old redirect id
			$oid=@$_POST['old_name'];

			if(!empty($oid) && isset($pr->db[$oid])) {
				unset($pr->id[$oid]);
			}

			$goto=trim($_POST['goto']);
			$name=trim($_POST['name']);

			if(!entry_exists($goto) || empty($goto)) {
				$s=-5;
				$nor=true;
			} elseif($pr->cleanId($name)!=$name || empty($name)) {
				$s=-6;
				$nor=true;
			} else {
				$pr->db[$name]=array(
					'entry'=>$goto,
					'comments'=>isset($_POST['incomm']),
					'httpredirect'=>isset($_POST['httpredirect']),
				);

				$s=$pr->savedb() ? 1 : -1;
			}

			$this->smarty->assign('success', $s);
			if(@$nor) {
				return $this->main();
			} else {
				return 2;
			}
		}

		/**
		 * The handler for the cancel buttons.
		 *
		 * @return integer: The panel redirection option
		 */
		function oncancel() {
			return 2;
		}

	}

	admin_addpanelaction('plugin', 'redirect', true);
}

# Redirect field in entry editor, compatible only with FP >= 0.1010
if(version_compare(SYSTEM_VER, '0.1010', '>=')==1 && defined('MOD_ADMIN_PANEL')) {
	/**
	 * plugin_redirect_entry
	 *
	 * This class is used to integrate the plugin with
	 * the entry editor of Flatpress.
	 *
	 * @author: Piero VDFN <vogliadifarniente@gmail.com>
	 * @license: GNU GPL v2
	 */
	class plugin_redirect_entry {

		/**
		 * The handler of the hook simple_edit_form.
		 *
		 * This function prints the panel of redirect options.
		 */
		function simple() {
			@$id=$_REQUEST['entry'];

			$pr=&$this->plugin_redirect;
			$pr->loaddb();

			// Get the old redirect (if exists) and comments
			$old='';
			$comm='';

			if(count($pr->db) && !empty($id)) {
				foreach((array)$pr->db as $r=>$params) {
					if($params['entry']==$id) {
						$old=$r;
						$comm=(bool)$params['comments'];
						$http=(bool)$params['httpredirect'];
					}
				}
			}

			if(!empty($_REQUEST['pl_redirect'])) {
				$old=$_REQUEST['pl_redirect'];
			}

			// Assign to Smarty the vars and print the form
			$rlang=lang_load('plugin:redirect');
			$smarty=&$this->smarty;
			$smarty->assign('old', $old);
			$smarty->assign('comm', $comm);
			$smarty->assign('http', $http);
			$smarty->assign('rlang', $rlang['admin']['plugin']['redirect']);
			$smarty->display('plugin:redirect/simple.redirect');
		}

		/**
		 * The handler for the publish_post hook.
		 *
		 * This function saves the redirect.
		 *
		 * @param string $id: the entry ID
		 * @param array $entry: the entry data
		 * @returns boolean: true, always
		 */
		function save($id, $entry) {
			$pr=&$this->plugin_redirect;
			$pr->loaddb();

			// Delete old redirects
			foreach((array)$pr->db as $name=>$value) {
				if($value['entry']==$id) {
					unset($pr->db[$name]);
				}
			}

			// Add the new redirect
			if(!empty($_POST['pl_redirect'])) {
				$name=$pr->cleanId($_POST['pl_redirect']);
				$pr->db[$name]=array(
					'entry'=>$id,
					'comments'=>isset($_POST['pl_redirect_comm']),
					'httpredirect'=>isset($_POST['pl_redirect_http']),
				);
			}

			// Save redirects
			$pr->savedb();

			// Return true in every case
			return true;
		}

		/**
		 * This is the constructor of the class.
		 *
		 * It saves by reference the plugin_redirect class and
		 * it sets some hooks.
		 *
		 * @param object $plugin_redirect: the plugin_redirect instance
		 * @param object $smarty: the Smarty instance
		 */
		function plugin_redirect_entry(&$plugin_redirect, &$smarty) {
			$this->plugin_redirect=&$plugin_redirect;
			$this->smarty=&$smarty;

			add_filter('simple_edit_form', array(&$this, 'simple'));
			add_filter('publish_post', array(&$this, 'save'), 10, 2);
		}

	}

	global $smarty;
	new plugin_redirect_entry($plugin_redirect, $smarty);
}